//
//  PhotoShowView.m
//  封装相册获取
//
//  Created by 史长硕 on 2018/8/23.
//  Copyright © 2018年 史长硕. All rights reserved.
//

#import "PhotoShowView.h"
#import "HCMacro.h"
#import "ShowPhotoCollectionViewCell.h"
#import "HCToastView.h"
@interface PhotoShowView ()<UICollectionViewDataSource,UICollectionViewDelegate>
//视图
@property(nonatomic, strong)UICollectionView *photoCollectionView;//照片展示视图
@property(nonatomic, strong)PHCachingImageManager *imageManager;//苹果缓存图片机制
@property(nonatomic, strong)UIButton *backButton;//返回按钮
@property(nonatomic, strong)UIView *topView;//顶部视图
@property(nonatomic, strong)UILabel *titleLabel;//顶部标题视图
@property(nonatomic, strong)UIButton *selectButton;//右上角选择按钮
//数据
@property(nonatomic, strong)NSMutableArray *selectImageArray;//用户选择照片数组
@property(nonatomic, strong)NSMutableArray *photoArray;//用户全部照片数组
@property(nonatomic, assign)NSInteger nowShowNum;//当前页数

@end
@implementation PhotoShowView

#pragma mark --> life Cycle
-(instancetype)initWithFrame:(CGRect)frame{
    if (self = [super initWithFrame:frame]) {
        [self addSubview:self.photoCollectionView];
        [self addSubview:self.topView];
        [self.topView addSubview:self.backButton];
        [self.topView addSubview:self.titleLabel];
        [self.topView addSubview:self.selectButton];
    }
    return self;
}

#pragma mark --> event Methods

//加载数据
-(void)reloadAllPohotImageArray:(NSArray *)imgArray SelectImageArray:(NSArray *)array andNowIndex:(NSIndexPath *)index{
    if (_selectImageArray) {
        [_selectImageArray removeAllObjects];
        [_selectImageArray addObjectsFromArray:array];
    }else{
        _selectImageArray = [NSMutableArray arrayWithArray:array];
    }
    if (_photoArray) {
        [_photoArray removeAllObjects];
        [_photoArray addObjectsFromArray:imgArray];
    }else{
        _photoArray = [NSMutableArray arrayWithArray:imgArray];
    }
    [self.imageManager startCachingImagesForAssets:_photoArray targetSize:CGSizeMake(DeviceWidth, DeviceHeight-TopViewHeight) contentMode:PHImageContentModeAspectFill options:nil];
    [self.photoCollectionView reloadData];
    self.photoCollectionView.contentOffset = CGPointMake(index.row*DeviceWidth, 0);
    [self changeTopShowNum:index.row];
}

//选择视图方法
-(void)selectImageAction:(UIButton *)sender{
    sender.selected = !sender.selected;
    if (sender.selected) {
        if (_selectImageArray.count == _maxImgNum) {
            sender.selected = NO;
            [[HCToastView defaultView] postAlertWithCenterType:ToastCenterType_Middle andMessage:[NSString stringWithFormat:@"最多可选%ld张图片",_maxImgNum]];
        }else{
          [_selectImageArray addObject:_photoArray[_nowShowNum]];
        }
        
    }else{
        [_selectImageArray removeObject:_photoArray[_nowShowNum]];
    }
}

//点击返回按钮
-(void)backAction{
    if (_delegate && [_delegate respondsToSelector:@selector(backSelectImgArray:)]) {
        [_delegate backSelectImgArray:_selectImageArray];
        WEAKSELF
        [UIView animateWithDuration:0.3 animations:^{
            STRONGSELF
            strongSelf.frame = CGRectMake(0, DeviceHeight, DeviceWidth, DeviceHeight);
        }];
    }

}

//顶部视图展示变化方法
-(void)changeTopShowNum:(NSInteger )number{
    _nowShowNum = number;
    self.titleLabel.text = [NSString stringWithFormat:@"%ld/%ld",number+1,_photoArray.count];
    if (_selectImageArray.count > 0) {
        if ([_selectImageArray containsObject:_photoArray[number]]) {
            self.selectButton.selected = YES;
        }else{
            self.selectButton.selected = NO;
        }
    }
}

#pragma mark --> collectionView 代理协议

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section{
    return _photoArray.count;
}

- (__kindof UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath{
    ShowPhotoCollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"show" forIndexPath:indexPath];
    [cell reloadPhotoModel:_photoArray[indexPath.row]];
    return cell;
}

-(void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView{
    
    NSInteger index = scrollView.contentOffset.x/DeviceWidth;
    [self changeTopShowNum:index];
}
#pragma mark --> set UI

-(UICollectionView *)photoCollectionView{
    if (!_photoCollectionView) {
        UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc]init];
        layout.itemSize = CGSizeMake(DeviceWidth, DeviceHeight-TopViewHeight);
        layout.minimumLineSpacing = 0.0;
        layout.minimumInteritemSpacing = 0.0;
        layout.scrollDirection = UICollectionViewScrollDirectionHorizontal;
        
        _photoCollectionView = [[UICollectionView alloc]initWithFrame:CGRectMake(0, TopViewHeight, DeviceWidth, DeviceHeight-TopViewHeight) collectionViewLayout:layout];
        _photoCollectionView.dataSource = self;
        _photoCollectionView.delegate = self;
        _photoCollectionView.backgroundColor = [UIColor whiteColor];
        _photoCollectionView.showsVerticalScrollIndicator = NO;
        _photoCollectionView.showsHorizontalScrollIndicator = NO;
        _photoCollectionView.pagingEnabled = YES;
        [_photoCollectionView registerClass:[ShowPhotoCollectionViewCell class] forCellWithReuseIdentifier:@"show"];
    }
    return _photoCollectionView;
}

-(PHCachingImageManager *)imageManager{
    if (!_imageManager) {
        _imageManager = [[PHCachingImageManager alloc]init];
    }
    return _imageManager;
}

-(UIButton *)backButton{
    if (!_backButton) {
        _backButton = [[UIButton alloc]initWithFrame:CGRectMake(15, 32, 20, 20)];
        [_backButton setImage:[UIImage imageNamed:@"白色返回"] forState:UIControlStateNormal];
        [_backButton addTarget:self action:@selector(backAction) forControlEvents:UIControlEventTouchUpInside];
        _backButton.backgroundColor = UIColorFromRGB(0x32beff, 1.0);
    }
    return _backButton;
}

-(UILabel *)titleLabel{
    if (!_titleLabel) {
        _titleLabel = [[UILabel alloc]initWithFrame:CGRectMake(80, 27, DeviceWidth-160, 30)];
        _titleLabel.backgroundColor = UIColorFromRGB(0x32beff, 1.0);
        _titleLabel.textAlignment = NSTextAlignmentCenter;
        _titleLabel.textColor = [UIColor whiteColor];
    }
    return _titleLabel;
}

-(UIButton *)selectButton{
    if (!_selectButton) {
        _selectButton = [[UIButton alloc]initWithFrame:CGRectMake(DeviceWidth-45, 27, 30, 30)];
        [_selectButton setImage:[UIImage imageNamed:@"圆圈"] forState:UIControlStateNormal];
        [_selectButton setImage:[UIImage imageNamed:@"优惠券_勾选"] forState:UIControlStateSelected];
        [_selectButton addTarget:self action:@selector(selectImageAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _selectButton;
}


-(UIView *)topView{
    if (!_topView) {
        _topView = [[UIView alloc]initWithFrame:CGRectMake(0, 0, DeviceWidth, 64)];
        _topView.backgroundColor = UIColorFromRGB(0x32beff, 1.0);
    }
    return _topView;
}

@end
