//
//  PhotoView.m
//  封装相册获取
//
//  Created by 史长硕 on 2018/8/22.
//  Copyright © 2018年 史长硕. All rights reserved.
//

#import "PhotoView.h"
#import "HCMacro.h"
#import "PhotoCollectionViewCell.h"
#import <Photos/Photos.h>
#import "HCToastView.h"
#import "PhotoBottomView.h"
#import "PhotoShowView.h"
#import "AlertErrorView.h"
@interface PhotoView ()<UICollectionViewDataSource,UICollectionViewDelegate,SelectImageDelegate,BackSelectImageDelegate>
//视图
@property(nonatomic, strong)UICollectionView *photoCollectionView;//照片展示视图
@property(nonatomic, strong)PHCachingImageManager *imageManager;//苹果缓存图片机制
@property(nonatomic, strong)PhotoBottomView *bottomView;//底部展示数量视图
@property(nonatomic, strong)PhotoShowView *showPhotoView;//浏览大图

//数据
@property(nonatomic, strong)NSMutableArray <PHAsset*> *photoArray;//照片数据源
@property(nonatomic, strong)NSMutableArray *selectImageArray;//选择照片数组
@property(nonatomic, assign)NSInteger selectImgNum;//所选照片数量
@end

@implementation PhotoView

#pragma mark --> life Cycle
-(instancetype)initWithFrame:(CGRect)frame{
    if (self = [super initWithFrame:frame]) {
        self.backgroundColor = [UIColor whiteColor];
        [self initializationData];
        [self addSubview:self.bottomView];
    }
    return self;
}

#pragma mark --> event Methods

//初始化数据
-(void)initializationData{
    [self addSubview:self.photoCollectionView];
    _selectImgNum = 0;
    //添加监听者
    [self addObserver:self forKeyPath:@"selectImgNum" options:NSKeyValueObservingOptionNew|NSKeyValueObservingOptionOld context:nil];
}

//加载数据
-(void)reloadPhotoInfoArray:(NSArray *)array{
    if (_photoArray) {
        [_photoArray removeAllObjects];
        [_photoArray addObjectsFromArray:array];
    }else{
        _photoArray = [NSMutableArray arrayWithArray:array];
    }
    if (_selectImageArray) {
        [_selectImageArray removeAllObjects];
    }else{
        _selectImageArray = [[NSMutableArray alloc]init];
    }
    self.imageManager = [[PHCachingImageManager alloc]init];
    if (IPHONE4 || iPhone5) {
        [self.imageManager startCachingImagesForAssets:_photoArray targetSize:CGSizeMake(DeviceWidth/3.0-20, DeviceWidth/3.0-20) contentMode:PHImageContentModeAspectFill options:nil];
    }else{
        [self.imageManager startCachingImagesForAssets:_photoArray targetSize:CGSizeMake((DeviceWidth-30)/4.0, (DeviceWidth-30)/4.0) contentMode:PHImageContentModeAspectFill options:nil];
    }
    [self.photoCollectionView reloadData];
}

//监听者方法
-(void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary<NSKeyValueChangeKey,id> *)change context:(void *)context{
    if ([keyPath isEqualToString:@"selectImgNum"]) {
        NSLog(@"old : %@  new : %@",[change objectForKey:@"old"],[change objectForKey:@"new"]);
    }
}

//展示浏览大图视图
-(void)showPhotoViewAction{
    WEAKSELF
    [UIView animateWithDuration:0.3 animations:^{
        STRONGSELF
        strongSelf.showPhotoView.frame = CGRectMake(0, 0, DeviceWidth, DeviceHeight);
    }];
}

//获取视图控制器
- (UIViewController *)viewController {
    
    for (UIView* next = [self superview]; next; next = next.superview) {
        UIResponder *nextResponder = [next nextResponder];
        if ([nextResponder isKindOfClass:[UIViewController class]]) {
            return (UIViewController *)nextResponder;
        }
    }
    return nil;
    
}

//判断该照片是否已选中
-(BOOL)isContainImage:(PHAsset *)asset{
    return [_selectImageArray containsObject:asset];
}

//提示错误信息
-(void)showError:(NSString *)error{
    [AlertErrorView showAlertView:[self viewController] andError:error andSelectNum:^(NSInteger number) {
        
    }];
}

#pragma mark --> collectionView 代理协议

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section{
    return _photoArray.count;
}

- (__kindof UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath{
    PhotoCollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"photo" forIndexPath:indexPath];
    cell.delegate = self;
    cell.isSelected = [self isContainImage:_photoArray[indexPath.row]];
    [cell reloadPhotoModel:_photoArray[indexPath.row]];
    return cell;
}

-(void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath{
    [self.showPhotoView reloadAllPohotImageArray:_photoArray SelectImageArray:_selectImageArray andNowIndex:indexPath];
    [self showPhotoViewAction];
}

#pragma mark --> 点击右上角选择按钮代理协议

-(void)sendSelectCell:(UICollectionViewCell *)cell andSelectBool:(BOOL)selected{
    NSIndexPath *index = [self.photoCollectionView indexPathForCell:cell];
    if (index) {
        if (selected) {
            if (_selectImgNum == _maxImgNum) {
                UIButton *btn = (UIButton *)[cell.contentView viewWithTag:60];
                btn.selected = NO;
                [[HCToastView defaultView] postAlertWithCenterType:ToastCenterType_Middle andMessage:[NSString stringWithFormat:@"亲，最多可以选择%ld张图片",_maxImgNum]];
            }else{
               self.selectImgNum ++;
               [_selectImageArray addObject:_photoArray[index.row]];
               [self.bottomView reloadImageNum:_selectImgNum];
            }
        }else{
            if ([_selectImageArray containsObject:_photoArray[index.row]]) {
                [_selectImageArray removeObject:_photoArray[index.row]];
                self.selectImgNum--;
                [self.bottomView reloadImageNum:_selectImgNum];
                
            }
        }
    }
    
}

#pragma mark --> 大图浏览代理协议
-(void)backSelectImgArray:(NSArray *)imgArray{
    
    [_selectImageArray removeAllObjects];
    [_selectImageArray addObjectsFromArray:imgArray];
    _selectImgNum =imgArray.count;
    if (_selectImageArray.count == 0) {
        self.bottomView.hidden = YES;
    }else{
        [self.bottomView reloadImageNum:_selectImgNum];
        self.bottomView.hidden = NO;
    }
    [self.photoCollectionView reloadData];
}

#pragma mark --> set UI

-(UICollectionView *)photoCollectionView{
    if (!_photoCollectionView) {
        UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc]init];
        if (IPHONE4 || iPhone5) {
            layout.itemSize = CGSizeMake(DeviceWidth/3.0-20, DeviceWidth/3.0-20);
        }else{
            layout.itemSize = CGSizeMake((DeviceWidth-30)/4.0, (DeviceWidth-30)/4.0);
        }
        layout.scrollDirection = UICollectionViewScrollDirectionVertical;
        layout.minimumLineSpacing = 10;
        layout.minimumInteritemSpacing = 10;
        
        _photoCollectionView = [[UICollectionView alloc]initWithFrame:CGRectMake(0, 0, DeviceWidth, DeviceHeight-TopViewHeight-50) collectionViewLayout:layout];
        _photoCollectionView.dataSource = self;
        _photoCollectionView.delegate = self;
        _photoCollectionView.backgroundColor = [UIColor whiteColor];
        [_photoCollectionView registerClass:[PhotoCollectionViewCell class] forCellWithReuseIdentifier:@"photo"];
    }
    return _photoCollectionView;
}

-(PhotoBottomView *)bottomView{
    if (!_bottomView) {
        _bottomView = [[PhotoBottomView alloc]initWithFrame:CGRectMake(0, DeviceHeight-TopViewHeight-50, DeviceWidth, 50)];
        WEAKSELF
        [_bottomView setBlock:^(UITouchTypeName name) {
            STRONGSELF
            if (name == TouchPreView) {
              NSIndexPath *index = [NSIndexPath indexPathForRow:0 inSection:0];
              [strongSelf.showPhotoView reloadAllPohotImageArray:strongSelf.selectImageArray SelectImageArray:strongSelf.selectImageArray andNowIndex:index];
                [strongSelf showPhotoViewAction];
            }else if (name == TouchNext){
                if (strongSelf.selectImageArray.count < strongSelf.minImgNum) {
                    [strongSelf showError:[NSString stringWithFormat:@"亲，最低需要选择%ld张图片",strongSelf.minImgNum]];
                }else{
                    if (strongSelf.delegate && [strongSelf.delegate respondsToSelector:@selector(transmitSelectImgArray:)]) {
                        [strongSelf.delegate transmitSelectImgArray:strongSelf.selectImageArray];
                    }
                }
            }
        }];
        
    }
    return _bottomView;
}

-(PhotoShowView *)showPhotoView{
    if (!_showPhotoView) {
        _showPhotoView = [[PhotoShowView alloc]initWithFrame:CGRectMake(0, DeviceHeight, DeviceWidth, DeviceHeight)];
        _showPhotoView.maxImgNum = _maxImgNum;
        _showPhotoView.delegate = self;
        [[self viewController].navigationController.view addSubview:_showPhotoView];
    }
    return _showPhotoView;
}

@end
